/**********************************************************************
 * Macro qui assigne les deux dimensions de l'indice de          ******
 * dfavorisation qubcois de 1991 et la zone  partir du       ******
 * code postal et un code municipal  tout fichier administratif ******
 * comprenant au moins le code postal. On utilise une version    ******
 * simplifie du FCCP+ produit par Statistique Canada.           ******
 * 																 ******
 * Paramtres de la macro                                        ******
 *  - IN: fichier SAS d'entre auquel on veut ajouter l'indice   ******
 *        de dfavorisation (matriel et social)                 ******
 *  - PCODE: nom de la variable du code postal  6 positions     ******
 *  - MUNIC: nom de la variable du code municipal. Par dfaut,   ******
 *           0=l'attribution ne se fait qu' partir du code      ******
 *           postal seulement                                    ******
 *  - OUT: nom du fichier SAS de sortie identique  IN mais les  ******
 *         nouvelles variables et les indices de dfavorisation  ******
 * Auteur: Philippe Gamache, INSPQ                               ******
 * Dernire modification: 16 juin 2008                           *******
 * Modifications: Ajout de la variable DATE qui indique la       *******
 *                version du programme d'assignation             *******
 ***********************************************************************/

filename fccp "D:\Projets\Indice dfavorisation\Indice 1991\Assignation\Avril 2008\Canada\fccp.txt";
filename munic "D:\Projets\Indice dfavorisation\Indice 1991\Assignation\Avril 2008\Canada\munic.txt";

data fccp;
infile fccp;
input  munic 1-12 pcode $ 13-18 @19 propcp 5.2 propmunic 5.2 quintmat 33 quintsoc 34 centmat 35-37 centsoc 38-40
       quintmatRC 41 quintsocRC 42 quintmatZONE 43 quintsocZONE 44 quintmatRMR 45 quintsocRMR 46 zone 47;
run;

data munic;
  infile munic;
  input munic 1-7 mun96 8-14 mun01 15-21;
run;

proc sort data=fccp;by pcode;run;

%macro assignation91(in,pcode,munic,out);

data fccp;
   set fccp;
   nuniq = _n_;
run;

  data fichin;
    %if &munic ne 0 %then %do; length munic 8.; %end;
    set &in;
    numero=_n_;
	nba=ranuni(766);
  /*** Mettre ici le nom de la variable de code postal 
	   et du code municipal ***/
	pcode=compress(&pcode);
			%if &munic ne 0 %then 
     %do; 
       if substr(pcode,1,1) in ("A") then munic=1000000+mod(&munic,1000000);
       else if substr(pcode,1,1) in ("B") then munic=1200000+mod(&munic,1200000);
       else if substr(pcode,1,1) in ("C") then munic=1100000+mod(&munic,1100000);
       else if substr(pcode,1,1) in ("E") then munic=1300000+mod(&munic,1300000);
       else if substr(pcode,1,1) in ("G","H","J") then munic=2400000+mod(&munic,2400000);
       else if substr(pcode,1,1) in ("K","L","M","N","P") then munic=3500000+mod(&munic,3500000);
       else if substr(pcode,1,1) in ("R") then munic=4600000+mod(&munic,4600000);
       else if substr(pcode,1,1) in ("S") then munic=4700000+mod(&munic,4700000);
       else if substr(pcode,1,1) in ("T") then munic=4800000+mod(&munic,4800000);
       else if substr(pcode,1,1) in ("V") then munic=5900000+mod(&munic,5900000);
       else if substr(pcode,1,1) in ("X") then munic=6100000+mod(&munic,6100000);
       else if substr(pcode,1,1) in ("Y") then munic=6000000+mod(&munic,6000000);
     %end;
	keep numero nba pcode sexe %if &munic ne 0 %then %do; munic %end;;
   run;

%if &munic ne 0 %then %do;


proc sql;
     create table test as
     select fichin.pcode, fichin.munic, numero, nba, fccp.propmunic, fccp.nuniq,
	        fccp.quintmat, fccp.quintsoc, fccp.centmat, fccp.centsoc, fccp.zone,
			fccp.quintmatRC, fccp.quintsocRC, fccp.quintmatZONE, fccp.quintsocZONE, fccp.quintmatRMR, fccp.quintsocRMR
     from fichin, fccp
     where fichin.pcode=fccp.pcode and fichin.munic=fccp.munic
	 order by numero,nuniq; 
    quit;

  proc sort data=test;
    by pcode munic numero;
   run;

   /*** Choix de l'indice de dfavorisation si plus d'un indice
        par code postal ***/
  data test2;
    set test;
    by pcode munic numero;
	retain cumul find direct;
	if first.numero then do; find=0; cumul=0; 
                             if last.numero then direct=1; else direct=0;
                         end;
	if find = 0 then do;
	   cumul=cumul+(propmunic/100);
	   if nba < cumul then do; find=1; output; end;
      end;

   run; 

  proc sort data=test2;
    by numero;
   run;

  proc sort data=fichin;
    by numero;
   run; 

  data appmunic1 fichin;
    merge fichin(in=in1) test2(in=in2);
	by numero;
	if in1*in2 then output appmunic1;
	else if in1 then output fichin;
   run;

   %end;

   proc sql;
     create table test as
     select fichin.pcode, fichin.munic, numero, nba, fccp.propcp, fccp.nuniq,
	        fccp.quintmat, fccp.quintsoc, fccp.centmat, fccp.centsoc, fccp.zone,
			fccp.quintmatRC, fccp.quintsocRC, fccp.quintmatZONE, fccp.quintsocZONE, fccp.quintmatRMR, fccp.quintsocRMR
     from fichin, fccp
     where fichin.pcode=fccp.pcode
	 order by numero,nuniq; 
    quit;

  proc sort data=test;
    by pcode numero;
   run;

   /*** Choix de l'indice de dfavorisation si plus d'un indice
        par code postal ***/
  data test2;
    set test;
    by pcode numero;
	retain cumul find direct;
	if first.numero then do; find=0; cumul=0; 
                             if last.numero then direct=1; else direct=0;
                         end;
	if find = 0 then do;
	   cumul=cumul+(propcp/100);
	   if nba < cumul then do; find=1; output; end;
      end;

   run; 

  proc sort data=test2;
    by numero;
   run;

  proc sort data=fichin;
    by numero;
   run; 

  data appcp1 noapp1;
    merge fichin(in=in1) test2(in=in2);
	by numero;
	if in1*in2 then output appcp1;
	else if in1 then output noapp1;
   run;

data tout;
    set %if &munic ne 0 %then %do; 
                                appmunic1(in=in1)
                              %end;
	appcp1(in=in2) noapp1(in=in3);

	 if in3=1 then do; quintmat = 0; quintsoc = 0; centmat = 0; centsoc = 0; zone = 0;
     end;

     if quintmat=0 then groupe=3;
	 else if quintmat=. then groupe=2;
	 else groupe=1;

	keep numero groupe quintmat quintsoc centmat centsoc quintmatRC quintsocRC quintmatZONE quintsocZONE 
         quintmatRMR quintsocRMR zone;
   run;

  proc sort; by numero; run; 

  data fichin;
     set &in;
	 numero=_n_;
    run;

  data &out;
    merge fichin tout;
	by numero;
	DATE = '20080618';
	format groupe fgroupe.;
	drop numero;
   run;

quit;
%mend assignation91;

proc format;
  value fgroupe 1="Assignation russie"
  		        2="Appartient  un SD sans indice"
				3="Aucun appariement CP - SD - Indice";
 run;

 /***************************************************************
 * Exemple: %assignation(naiss96,cp,cod_mun,naiss96v5); *********
 * - Le fichier en entre est WORK.NAISS96              *********
 * - La variable pour le code postal est CP             *********
 * - La variable pour le code municipal est COD_MUN     *********
 *   (N.B.: on doit mettre 0 (zro) si on ne veut pas   *********
 *   faire l'assignation  l'aide du code municipal     *********
 * - Le fichier rsultant est WORK.NAISS96V2 (identique *********
 *   au fichier d'entre mais avec les indices de       *********
 *   dfavorisation en quintiles, quartiles et centiles *********
 *   et la variable ZONE)                               *********
 * - NOTE: Les rsultats peuvent varier si l'ordre du   *********
 *   fichier d'entre n'est pas la mme.                *********
 ****************************************************************/ 


 libname toto "c:\Documents and Settings\gamphi01\Mes documents\Projets\Indice dfavorisation\Assignation\Fichiers  assigner";

%assignation91(in=d1991,pcode=CPRESDCD,munic=CMUNRESDCD,out=d91);
